/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.task.project;

import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import nl.cloudfarming.client.geometry.model.GeometryContainer;
import nl.cloudfarming.client.geometry.model.Shape;
import nl.cloudfarming.client.geometry.model.Shape.ShapeType;
import nl.cloudfarming.client.task.model.Task;
import nl.cloudfarming.client.task.model.TaskManager;
import nl.cloudfarming.client.task.model.TreatmentZone;
import nl.cloudfarming.client.task.type.TaskDataObject;
import org.netbeans.api.project.Project;
import org.netbeans.api.project.ProjectInformation;
import org.netbeans.spi.project.ActionProvider;
import org.netbeans.spi.project.ProjectState;
import org.netbeans.spi.project.ui.support.DefaultProjectOperations;
import org.openide.filesystems.*;
import org.openide.loaders.DataObject;
import org.openide.loaders.DataObjectNotFoundException;
import org.openide.util.Exceptions;
import org.openide.util.ImageUtilities;
import org.openide.util.Lookup;
import org.openide.util.lookup.Lookups;

/**
 * Project for tasks
 *
 * @author Merijn Zengers & Gerben Feenstra
 */
public class TaskProject implements Project, GeometryContainer<TreatmentZone>, FileChangeListener {

    public static final String PROJECT_ICON = "nl/cloudfarming/client/task/project/TaskProject.png";
    private final FileObject projectDir;
    private final ProjectState state;
    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.task.project");
    private static final String IMPORTED_TASK = "ImportedTask";
    private Lookup lkp;
    private List<TreatmentZone> shapes;


    public TaskProject(FileObject projectDir, ProjectState state) {
        this.projectDir = projectDir;
        this.state = state;
    }

    @Override
    public FileObject getProjectDirectory() {
        return projectDir;
    }

    @Override
    public Lookup getLookup() {
        if (lkp == null) {
            lkp = Lookups.fixed(
                    this,
                    state, //allow outside code to mark the project as needing saving
                    new ActionProviderImpl(), //Provides standard actions like Build and Clean
                    new TaskLogicalView(this), //Logical view of project implementation
                    new TaskProjectInfo());
        }
        return lkp;
    }

    public FileObject getTasksFolder(boolean create) {
        FileObject result = projectDir.getFileObject(TaskProjectFactory.TASKS_DIR);
        if (result == null && create) {
            try {
                result = projectDir.createFolder(TaskProjectFactory.TASKS_DIR);
            } catch (IOException ioe) {
                Exceptions.printStackTrace(ioe);
            }
        }
        return result;
    }

    @Override
    public List<TreatmentZone> getShapes() {
        if (shapes == null) {
            FileObject tasksDir = getTasksFolder(true);
            tasksDir.addFileChangeListener(this);
            TaskManager taskManager = Lookup.getDefault().lookup(TaskManager.class);

            assert (taskManager != null);

            shapes = new ArrayList<>();

            for (FileObject taskFile : tasksDir.getChildren()) {
                Task task = taskManager.load(taskFile);
                shapes.addAll(task.getTreatmentZones());
            }
        }
        return shapes;
    }

    @Override
    public String toString() {
        return projectDir.getName();
    }

    @Override
    public String getDisplayName() {
        return projectDir.getName();
    }

    @Override
    public void importShapes(List<Shape> shapes) {

        List<TreatmentZone> treatmentZones = new ArrayList<>();

        for (Shape shape : shapes) {
            TreatmentZone tz = new TreatmentZone();
            tz.setGeometry(shape.getGeometry());
            tz.setName(shape.getName());
            tz.setValue(shape.getValue());
            treatmentZones.add(tz);
        }

        FileObject tasksFolder = getTasksFolder(true);

        // Create Task for newly imported shapes
        String name = FileUtil.findFreeFileName(tasksFolder, IMPORTED_TASK, Task.EXT);
        LOGGER.log(Level.FINEST, "found free filename: {0}", name);
        FileObject taskFile;
        try {
            taskFile = tasksFolder.createData(name, Task.EXT);
            Task task = new Task();
            task.setTreatmentZones(treatmentZones);

            TaskManager taskManager = Lookup.getDefault().lookup(TaskManager.class);
            taskManager.save(task, taskFile);
        } catch (IOException ex) {
            Exceptions.printStackTrace(ex);
        }
    }

    @Override
    public void importSources(DataObject dataObject) {
    }

    @Override
    public ShapeType getShapeType() {
        return ShapeType.TREATMENT_ZONE;
    }

    @Override
    public boolean supports(ShapeType shapeType) {
        return shapeType.equals(ShapeType.TREATMENT_ZONE);
    }

    @Override
    public void fileFolderCreated(FileEvent fe) {
        if (!fe.getFile().isFolder()) {
            try {
                DataObject newDataObject = DataObject.find(fe.getFile());
                if (newDataObject instanceof TaskDataObject) {
                    Task task = newDataObject.getLookup().lookup(Task.class);
                    for (TreatmentZone treatmentZone : task.getTreatmentZones()) {
                        shapes.add(treatmentZone);
                    }
                }
            } catch (DataObjectNotFoundException ex) {
                //Do nothing could happen. Maybe a folder is created
            }
        }
    }

    @Override
    public void fileDataCreated(FileEvent fe) {
    }

    @Override
    public void fileChanged(FileEvent fe) {
    }

    @Override
    public void fileDeleted(FileEvent fe) {
        if (!fe.getFile().isFolder()) {
            try {
                DataObject newDataObject = DataObject.find(fe.getFile());
                if (newDataObject instanceof TaskDataObject) {
                    Task task = newDataObject.getLookup().lookup(Task.class);
                    for (TreatmentZone treatmentZone : task.getTreatmentZones()) {
                        shapes.remove(treatmentZone);
                    }
                }
            } catch (DataObjectNotFoundException ex) {
                //Do nothing could happen. Maybe a folder is created
            }
        }
    }

    @Override
    public void fileRenamed(FileRenameEvent fe) {
    }

    @Override
    public void fileAttributeChanged(FileAttributeEvent fe) {
    }

    private final class ActionProviderImpl implements ActionProvider {

        private String[] supported = new String[]{
            ActionProvider.COMMAND_DELETE,
            ActionProvider.COMMAND_COPY,};

        @Override
        public String[] getSupportedActions() {
            return supported;
        }

        @Override
        public void invokeAction(String string, Lookup lookup) throws IllegalArgumentException {
            if (string.equalsIgnoreCase(ActionProvider.COMMAND_DELETE)) {
                DefaultProjectOperations.performDefaultDeleteOperation(TaskProject.this);
            }
            if (string.equalsIgnoreCase(ActionProvider.COMMAND_COPY)) {
                DefaultProjectOperations.performDefaultCopyOperation(TaskProject.this);
            }
        }

        @Override
        public boolean isActionEnabled(String command, Lookup lookup) throws IllegalArgumentException {
            switch (command) {
                case ActionProvider.COMMAND_DELETE:
                    return true;
                case ActionProvider.COMMAND_COPY:
                    return true;
                default:
                    throw new IllegalArgumentException(command);
            }
        }
    }

    private final class TaskProjectInfo implements ProjectInformation {

        @Override
        public Icon getIcon() {
            return new ImageIcon(ImageUtilities.loadImage(
                    PROJECT_ICON));
        }

        @Override
        public String getName() {
            return getProjectDirectory().getName();
        }

        @Override
        public String getDisplayName() {
            return getName();
        }

        @Override
        public void addPropertyChangeListener(PropertyChangeListener pcl) {
            //do nothing, won't change
        }

        @Override
        public void removePropertyChangeListener(PropertyChangeListener pcl) {
            //do nothing, won't change
        }

        @Override
        public Project getProject() {
            return TaskProject.this;
        }
    }
}
